void UseTheModule(FSSpec *theFS, Boolean useDefault);
OSErr GetNamedModuleFS(FSSpec *theFS, Str255 theName);
OSErr BuildModulesList(MenuHandle theMenu);
OSErr BuildReferencesList(MenuHandle theMenu, FSSpec theFS);
OSErr DoNewModule(void);
OSErr OpenTheModule(FSSpec *theFS, Boolean newModule, Boolean useOldFile);
Boolean ReferenceNameExistsQQ(FSSpec theFS, Str255 refName);
OSErr DeleteOneReference(FSSpec theFS, Str255 refName);
OSErr DeleteOneInstantiation(FSSpec theFS, Str255 refName, short stringIndex);
OSErr AddOneReference(FSSpec theFS, Str255 refName);
OSErr AddOneInstantiation(FSSpec theFS, Str255 refName, Str255 instantName);
OSErr ReplaceOneReference(FSSpec theFS, Str255 oldRef, Str255 newRef);
OSErr ReplaceOneInstantiation(FSSpec theFS, Str255 oldRef, short stringIndex, Str255 newStr);
OSErr BuildTEFromResourceFile(FSSpec theFS, WindowPtr theWindow);
OSErr AddInstantiationsFromFile(FSSpec theFS, WindowPtr theWindow, Str255 refName);
